#include <algorithm>
#include <iostream>
#include <vector>
#include <random>
#include <map>
#include <set>

#include "islands.h"
//#include "Lgrader.cpp"

using namespace std;

mt19937 rnd(22);
const int MAXN = 1e4 + 5;

set <int> possibleStates[MAXN];

void printPartition(vector <vector <int>> v)
{
    for(vector <int> &curr: v)
    {
        cout << "{";
        for(int x: curr) cout << x << ", ";
        cout << "}, ";
    }
    cout << '\n';
}

int askOne(int ind, int n)
{
    //cout << "ind: " << ind << '\n';
    vector <vector <int>> parts = {{ind}};

    for(int i = 0;i<n;)
    {
        vector <int> curr = {};
        for(int done = 0;i<n && done<2;i++)
        {
            if(i==ind) continue;

            done++;
            curr.push_back(i);
        }

        if(curr.size()==2) parts.push_back(curr);
        else if(curr.size()==1) parts.back().push_back(curr[0]);
    }

    //printPartition(parts);

    vector <vector<int>> res = ask(parts);
    for(vector <int> &v: res)
    {
        if(v.size()==1) return v[0];
    }
}

void init(int n, vector <int> &allNums)
{
    for(int i = 0;i<n;i++)
        possibleStates[i].clear();

    allNums.clear();
    for(int i = 0;i<n;i++) allNums.push_back(i);
}

void applyInfo(int ind, vector <int> v)
{
    bool wasEmpty = possibleStates[ind].empty();
    set <int> s;

    for(int x: v)
    {
        if(wasEmpty==true || possibleStates[ind].count(x)==true)
            s.insert(x);
    }

    possibleStates[ind] = s;
}

bool checkReady(int n)
{
    for(int i = 0;i<n;i++)
    {
        if(possibleStates[i].size()!=1) return false;
    }

    return true;
}

void allign(vector <vector <int>> &parts, vector <vector <int>> &res, int n)
{
    vector <vector <int>> newRes;
    int *num2Part = new int[n];

    for(int i = 0;i<res.size();i++)
    {
        for(int x: res[i])
            num2Part[x] = i;
    }

    for(int i = 0;i<parts.size();i++)
    {
        int transformedVal = askOne(parts[i][0], n);
        newRes.push_back(res[num2Part[transformedVal]]);
    }

    res = newRes;
}

vector <int> solve(int n)
{
    vector <int> allNums;
    init(n, allNums);

    int partSz = 2;
    while(true)
    {
        shuffle(allNums.begin(), allNums.end(), rnd);

        vector <vector <int>> parts;
        for(int i = 0;i<n;)
        {
            int startInd = i;
            parts.push_back({});

            for(;i<n && i<startInd+partSz;i++)
                parts.back().push_back(allNums[i]);
        }

        //cout << "parts: ";printPartition(parts);

        vector <vector <int>> res = ask(parts);
        allign(parts, res, n);

        //cout << "res: ";printPartition(res);

        for(int i = 0;i<parts.size();i++)
        {
            for(int x: parts[i])
            {
                applyInfo(x, res[i]);
            }
        }

        if(checkReady(n)==true) break;
    }

    vector <int> ans(n);
    for(int i = 0;i<n;i++) ans[i] = *possibleStates[i].begin();

    return ans;
}

