#include <algorithm>
#include <iostream>
#include <vector>

#include "ctree.h"
//#include "Lgrader.cpp"

using namespace std;

const int MAXN = 3e5 + 5;

int treeSize[MAXN];
vector <int> graph[MAXN];

int centroid;
vector <pair <int, int>> centroidChildren;

void dfsInit(int x, int last)
{
    treeSize[x] = 1;
    for(int y: graph[x])
    {
        if(y==last) continue;

        dfsInit(y, x);
        treeSize[x] += treeSize[y];
    }
}

int findCentroid(int n)
{
    dfsInit(0, -1);
    int centroid = 0, last = -1;

    while(true)
    {
        bool update = false;
        for(int y: graph[centroid])
        {
            if(y==last) continue;
            if(treeSize[y]>treeSize[0]/2)
            {
                last = centroid;
                centroid = y;

                update = true;
                break;
            }
        }

        if(update==false) break;
    }

    return centroid;
}

void loadChildren(int x, int last, vector <pair <int, int>> &children)
{
    for(int y: graph[x])
    {
        if(y==last) continue;
        children.push_back({treeSize[y], y});
    }
    sort(children.begin(), children.end(), greater<pair <int, int>>());
}

void init(int n, vector <int> &X, vector <int> &Y)
{
    for(int i = 0;i<n;i++)
    {
        graph[i].clear();
    }

    for(int eInd = 0;eInd<X.size();eInd++)
    {
        graph[X[eInd]].push_back(Y[eInd]);
        graph[Y[eInd]].push_back(X[eInd]);
    }

    centroid = findCentroid(n);
    dfsInit(centroid, -1);
    //cout << "centroid: " << centroid << '\n';

    centroidChildren.clear();
    loadChildren(centroid, -1, centroidChildren);
}

int getCnt(int sz, int root, int n)
{
    int cnt = 0;
    for(pair <int, int> c: centroidChildren)
    {
        if(sz<=n/2) break;

        if(c.second!=root)
        {
            cnt++;
            sz -= c.first;

            //cout << "taking " << c.second << " with mass " << c.first << '\n';
        }
    }

    return cnt;
}

void arbeit(int x, int last, int root, int pathLen, int n, vector <int> &testcaseAns)
{
    int treeSizeLast = n - treeSize[x];
    //cout << x << ": " << treeSizeLast << " | " << root << '\n';

    if(treeSizeLast>=n/2)
    {
        //vector <pair <int, int>> children;
        //loadChildren(x, last, children);

        int cnt1 = getCnt(treeSizeLast, root, n);
        int cnt2 = 1 + getCnt(treeSizeLast-pathLen, root, n);

        testcaseAns[x] = min(cnt1, cnt2);
        //cout << x << " -> " << min(cnt1, cnt2) << " " << treeSizeLast << '\n';
    }

    for(int y: graph[x])
    {
        if(y==last) continue;
        arbeit(y, x, root, pathLen+1, n, testcaseAns);
    }
}

vector <int> solve(int n, vector <int> X, vector <int> Y)
{
    init(n, X, Y);
    vector <int> testcaseAnswer(n);

    testcaseAnswer[centroid] = 0;
    for(int x: graph[centroid])
        arbeit(x, centroid, x, 0, n, testcaseAnswer);

    return testcaseAnswer;
}


