#include <algorithm>
#include <iostream>
#include <vector>
#include <set>

#include "ctree.h"
//#include "Lgrader.cpp"

using namespace std;

const int MAXN = 3e5 + 5;

int treeSize[MAXN];
vector <int> graph[MAXN];

int centroid;
set <pair <int, int>> activeChildren;
vector <pair <int, int>> centroidChildren;

void dfsInit(int x, int last)
{
    treeSize[x] = 1;
    for(int y: graph[x])
    {
        if(y==last) continue;

        dfsInit(y, x);
        treeSize[x] += treeSize[y];
    }
}

int findCentroid(int n)
{
    dfsInit(0, -1);
    int centroid = 0, last = -1;

    while(true)
    {
        bool update = false;
        for(int y: graph[centroid])
        {
            if(y==last) continue;

            //cout << treeSize[y] << " " << y << '\n';
            if(treeSize[y]>treeSize[0]/2)
            {
                last = centroid;
                centroid = y;

                update = true;
                break;
            }
        }

        if(update==false) break;
    }

    return centroid;
}

void loadChildren(int x, int last, vector <pair <int, int>> &children)
{
    for(int y: graph[x])
    {
        if(y==last) continue;
        children.push_back({treeSize[y], y});
    }
    sort(children.begin(), children.end(), greater<pair <int, int>>());
}

void init(int n, vector <int> &X, vector <int> &Y)
{
    for(int i = 0;i<150;i++)
    {
        graph[i].clear();
    }

    for(int eInd = 0;eInd<X.size();eInd++)
    {
        graph[X[eInd]].push_back(Y[eInd]);
        graph[Y[eInd]].push_back(X[eInd]);
    }

    centroid = findCentroid(n);
    dfsInit(centroid, -1);
    //cout << "centroid: " << centroid << '\n';

    //centroidChildren.clear();
    //loadChildren(centroid, -1, centroidChildren);

    //activeChildren.clear();
    //for(auto x: centroidChildren) activeChildren.insert(x);
}


int getCnt1(int sz, int root, int n)
{
    //return -1;

    int cnt = 0;
    auto it = activeChildren.end();

    while(sz>n/2)
    {
        if(it==activeChildren.begin())
        {
            cnt = MAXN;
            break;
        }

        it--;
        //cout << it->second << " " << it->first << '\n';

        cnt++;
        sz -= it->first;
    }

    return cnt;
}

int getCnt2(int sz, int root, int n)
{
    //return -1;

    int cnt = 0;
    for(pair <int, int> c: centroidChildren)
    {
        if(sz<=n/2) break;

        if(c.second!=root)
        {
            cnt++;
            sz -= c.first;

            //cout << "taking " << c.second << " with mass " << c.first << '\n';
        }
    }

    return cnt;
}

void arbeit(int x, int last, int root, int pathLen, int n, vector <int> &testcaseAns)
{
    int treeSizeLast = n - treeSize[x];
    //cout << x << ": " << treeSizeLast << " | " << root << " | " << pathLen << '\n';

    if(pathLen+1>n/2)
    {
        cout << 1/0 << '\n';
    }

    if(treeSizeLast>n/2)
    {
        //vector <pair <int, int>> children;
        //loadChildren(x, last, children);

        int cnt1 = getCnt1(treeSizeLast, root, n);
        int cnt2 = 1 + getCnt2(treeSizeLast-pathLen, root, n);

        testcaseAns[x] = min(cnt1, cnt2);
        //cout << x << " -> " << min(cnt1, cnt2) << " " << treeSizeLast << '\n';
    }
    else
    {
        testcaseAns[x] = 0;
    }

    for(int y: graph[x])
    {
        if(y!=last)
            activeChildren.insert({treeSize[y], y});
    }

    for(int y: graph[x])
    {
        if(y==last) continue;

        activeChildren.erase({treeSize[y], y});
        arbeit(y, x, root, pathLen+1, n, testcaseAns);
        activeChildren.insert({treeSize[y], y});
    }

    for(int y: graph[x])
    {
        if(y!=last)
            activeChildren.erase({treeSize[y], y});
    }
}

void dfs(int x, int pathLen, int last, int n)
{
    if(pathLen+1>n/2)
    {
        cout << 1/0 << '\n';
        exit(-1);
    }

    for(int y: graph[x])
    {
        if(y==last) continue;
        dfs(y, pathLen+1, x, n);
    }
}

vector <int> solve(int n, vector <int> X, vector <int> Y)
{
    init(n, X, Y);
    vector <int> testcaseAnswer(n);

    for(int x: graph[centroid])
    {
        dfs(x, 0, centroid, n);
    }
    /*
    testcaseAnswer[centroid] = 0;
    for(int x: graph[centroid])
    {
        activeChildren.erase({treeSize[x], x});
        arbeit(x, centroid, x, 0, n, testcaseAnswer);
        activeChildren.insert({treeSize[x], x});
    }
    */


    return testcaseAnswer;
}


